﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Astronomy
{
	/// <summary>
	/// Небесен обект
	/// </summary>
	public abstract class CelestialObject : IDumpable
	{
		#region Alternative 1. Свойство установявано само в този клас

		//private string _name;

		//public string Name
		//{
		//    get { return this._name; }
		//    private set { this._name = value; }
		//}

		#endregion

		#region Alternative 2. Автоматично свойство установявано само в този клас

		//public string Name { get; private set; }

		#endregion

		#region Alternative 3. Свойство с частно поле установявано само от конструктора

		private readonly string _name;

		/// <summary>
		/// Наименование
		/// </summary>
		public string Name
		{
			get { return this._name; }
		}

		#endregion

		/// <summary>
		/// Текуща скорост
		/// </summary>
		public long CurrentVelocity { get; set; }

		/// <summary>
		/// Конструира небесен обект.
		/// </summary>
		/// <param name="name">Наименование на небесния обект</param>
		/// <remarks>Конструкторът е "protected", тъй като е конструктор на абстрактен клас и, като такъв, може да бъде извикван само от наследниците.</remarks>
		protected CelestialObject(string name)
		{
			this._name = name;
		}

		/// <summary>
		/// Разпечатва съдържанието на обекта.
		/// </summary>
		public abstract void Dump();

		/// <summary>
		/// Визуализира стрингово представяне на небесен обект.
		/// </summary>
		/// <returns>стрингово представяне на небесен обект</returns>
		public override string ToString()
		{
			return "Type = " + this.GetType() + " ; Name = " + this._name;
		}
	}
}
