﻿using System;
using System.Collections.Generic;

namespace Astronomy
{
	/// <summary>
	/// Планета - небесен обект (CelestialObject), спътник (IMoon) със спътници (IPrimary)
	/// </summary>
	public class Planet : CelestialObject, IPrimary, IMoon
	{
		private readonly IPrimary _primary;
		private readonly List<IMoon> _moons;

		#region IDumpable Members

		public override void Dump()
		{
			Console.WriteLine("Begin Planet '{0}'.", Name);

			Console.WriteLine("Moons:");
			foreach (IMoon moon in this._moons)
			{
				moon.Dump();
			}

			Console.WriteLine("End Planet '{0}'.", Name);
		}

		#endregion

		#region IPrimary Members

		public List<IMoon> Moons
		{
			get { return _moons; }
		}

		#endregion

		#region IMoon Members

		public IPrimary Primary
		{
			get { return _primary; }
		}

		public long DistanceToPrimary { get; set; }

		public TimeSpan RevolutionPeriod { get; set; }

		#endregion

		/// <summary>
		/// Има или не атмосфера
		/// </summary>
		public bool HasAtmosphere { get; set; }

		/// <summary>
		/// Завърта планетата.
		/// </summary>
		public void Spin()
		{
			throw new NotImplementedException();
		}

		public Planet(string name, IPrimary primary)
			: base(name)
		{
			this._primary = primary;
			this._moons = new List<IMoon>();
		}
	}
}
