﻿using System;

namespace DelegatesAndEvents
{
	/// <summary>
	/// Засича наличието на егоцентризъм в подаден текст.
	/// </summary>
	public class EgocentricDetector
	{
		private static readonly char[] delimiters = new char[]
		{
			' ', '\n', '\t', '?', '!', '.', ',', ';', '-',
		};
		private string _text;

		/// <summary>
		/// Промяна на състоянието на детектора.
		/// </summary>
		public event EgocentricDetectorStatusDelegate StatusChanged;

		/// <summary>
		/// Създава детектор на егоцентризъм за определен текст.
		/// </summary>
		/// <param name="text">Текст, в който ще се търси егоцентризъм</param>
		public EgocentricDetector(string text)
		{
			this._text = text;
		}

		/// <summary>
		/// Обработва текст
		/// </summary>
		/// <param name="callback"></param>
		public void Process(EgocentricDetectorDelegate callback)
		{
			_text = _text.ToLower();
			// разделяне на текста на думи
			string[] words = _text.Split(delimiters,
				StringSplitOptions.RemoveEmptyEntries);

			// обхождане на всяка дума поотделно
			for (int i = 0; i < words.Length; i++)
			{
				#region Известяване за промяна на състоянието

				// Ако има абонати за събитието, ги уведомява
				if (StatusChanged != null)
					StatusChanged.Invoke(i + 1, words.Length);

				#endregion

				string word = words[i];

				if (word == "аз"
					|| word.EndsWith("ам")
					|| word.EndsWith("ям")
					)
				{
					#region Алтернатива 1. Неявно извикване

					//callback(word);

					#endregion

					#region Алтернатива 2. Явно синхронно извикване (с изчакване)

					callback.Invoke(word);

					#endregion

					#region Алтернатива 3. Явно асинхронно извикване (без изчакване)

					//callback.BeginInvoke(word, null, null);

					#endregion
				}
			}
		}
	}
}
