﻿using System;

namespace Persons
{
	public abstract class Person
	{
		/// <summary>
		/// Взима цялото име на лицето.
		/// </summary>
		/// <returns>Цяло име на лицето</returns>
		public abstract string GetFullName();

		/// <summary>
		/// Представя се.
		/// </summary>
		public void Introduce()
		{
			string fullName = GetFullName();
			Console.WriteLine("Hi, I am {0}!", fullName);
		}

		/// <summary>
		/// Виртуален метод, който се извиква преди реалното
		/// записване на обекта в база данни. Наследниците могат
		/// да презапишат този метод, в случай, че искат да направят
		/// нещо преди записа.
		/// </summary>
		protected virtual void OnSaving()
		{
		}

		/// <summary>
		/// Записва обекта в база данни.
		/// </summary>
		public void Save()
		{
			// Извиква виртуален метод преди започване
			// на записа в база данни.
			OnSaving();

			Console.WriteLine("[Save to Database]");

			// Извиква виртуален метод след приключване
			// на записа в база данни.
			OnSaved();
		}

		/// <summary>
		/// Виртуален метод, който се извиква след реалното
		/// записване на обекта в база данни. Наследниците могат
		/// да презапишат този метод, в случай, че искат да направят
		/// нещо след записа.
		/// </summary>
		protected virtual void OnSaved()
		{
		}
	}
}
